@@ -82,6 +82,8 @@ class TestBuscoConfig(unittest.TestCase):
                 "use_augustus",
                 "batch_mode",
                 "tar",
+                "contig_break",
+                "scaffold_composition",
             ],
             "etraining": ["path", "command"],
             "gff2gbSmallDNA.pl": ["path", "command"],
@@ -98,14 +100,14 @@ class TestBuscoConfig(unittest.TestCase):
     def test_read_config_file(self):
         config = BuscoConfig.BaseConfig()
         config.conf_file = self.base_config
-        config._load_config_file()
+        config._load_config_file(config.conf_file)
         self.assertIn("busco_run", config.sections())
 
     def test_read_config_file_ioerror(self):
         with self.assertRaises(BuscoConfig.BatchFatalError):
             config = BuscoConfig.BaseConfig()
             config.conf_file = "/path/not/found"
-            config._load_config_file()
+            config._load_config_file(config.conf_file)
 
     def test_read_config_file_parseerror(self):
         config_path = "tests/config_parseerror_test.ini"
@@ -116,7 +118,7 @@ class TestBuscoConfig(unittest.TestCase):
         with self.assertRaises(BuscoConfig.BatchFatalError):
             config = BuscoConfig.BaseConfig()
             config.conf_file = config_path
-            config._load_config_file()
+            config._load_config_file(config.conf_file)
         os.remove(config_path)
 
     def test_read_config_file_duplicateerror(self):
@@ -128,7 +130,7 @@ class TestBuscoConfig(unittest.TestCase):
         with self.assertRaises(BuscoConfig.BatchFatalError):
             config = BuscoConfig.BaseConfig()
             config.conf_file = config_path
-            config._load_config_file()
+            config._load_config_file(config.conf_file)
         os.remove(config_path)
 
     def test_config_update_args_bool(self):
@@ -151,7 +153,7 @@ class TestBuscoConfig(unittest.TestCase):
             "evalue": "0.01",
             "in": "input_file",
             "limit": "1",
-            "lineage_dataset": "test",
+            "lineage_dataset": "test_odb10",
             "mode": "test",
             "out": "test",
             "out_path": "test",
@@ -172,10 +174,7 @@ class TestBuscoConfig(unittest.TestCase):
             "datasets_version": "odb10",
             "download_base_url": "https://busco-data.ezlab.org/v5/data/",
             "download_path": os.path.join(os.getcwd(), "busco_downloads"),
-            "evalue": "0.001",
             "force": False,
-            "limit": "3",
-            "long": False,
             "offline": False,
             "out_path": os.getcwd(),
             "quiet": False,
@@ -183,7 +182,9 @@ class TestBuscoConfig(unittest.TestCase):
             "update-data": False,
             "use_augustus": False,
         }
-        config = BuscoConfig.BuscoConfigMain(self.base_config, {})
+        config = BuscoConfig.BuscoConfigMain(
+            self.base_config, {"lineage_dataset": "test"}
+        )
         config.configure()
         config_default_filled = {
             key: config.get("busco_run", key) for key in correct_default_params
@@ -194,9 +195,13 @@ class TestBuscoConfig(unittest.TestCase):
             config_default_filled,
         )
 
+    @patch(
+        "busco.BuscoConfig.BuscoConfigMain.check_lineage_present",
+        side_effect=[False, False],
+    )
     @patch(
         "busco.BuscoConfig.BuscoConfigMain.getboolean",
-        side_effect=[True, False, False, True, False],
+        side_effect=[False, True, True, False, False, False, True, False],
     )
     def test_config_auto_lineage_settings(self, *args):
         for _ in range(2):
@@ -204,13 +209,54 @@ class TestBuscoConfig(unittest.TestCase):
             config.configure()
             self.assertEqual(config.get("busco_run", "auto-lineage"), "True")
 
-    @patch("busco.BuscoConfig.BuscoConfigMain.getboolean", return_value=True)
+    @patch(
+        "busco.BuscoConfig.BuscoConfigMain.check_lineage_present",
+        side_effect=[False],
+    )
+    @patch(
+        "busco.BuscoConfig.BuscoConfigMain.getboolean",
+        side_effect=[False, False, True, True, True],
+    )
     def test_config_auto_lineage_both_selected_warning(self, *args):
         with self.assertLogs(BuscoConfig.logger, "WARNING"):
             config = BuscoConfig.BuscoConfigMain(self.base_config, {})
             config.configure()
         self.assertEqual(config.get("busco_run", "auto-lineage-euk"), "False")
         self.assertEqual(config.get("busco_run", "auto-lineage-prok"), "False")
+        self.assertEqual(config.get("busco_run", "auto-lineage"), "True")
+
+    @patch(
+        "busco.BuscoConfig.BuscoConfigMain.check_lineage_present",
+        side_effect=[False],
+    )
+    @patch(
+        "busco.BuscoConfig.BuscoConfigMain.getboolean",
+        side_effect=[False, False, False, False],
+    )
+    def test_config_auto_lineage_none_selected_no_lineage_warning(self, *args):
+        with self.assertLogs(BuscoConfig.logger, "WARNING"):
+            config = BuscoConfig.BuscoConfigMain(self.base_config, {})
+            config.configure()
+        self.assertEqual(config.get("busco_run", "auto-lineage-euk"), "False")
+        self.assertEqual(config.get("busco_run", "auto-lineage-prok"), "False")
+        self.assertEqual(config.get("busco_run", "auto-lineage"), "True")
+
+    @patch(
+        "busco.BuscoConfig.BuscoConfigMain.check_lineage_present",
+        side_effect=[True, True, True],
+    )
+    @patch(
+        "busco.BuscoConfig.BuscoConfigMain.getboolean",
+        side_effect=[False, True, False, True, False, False, True],
+    )
+    def test_config_auto_lineage_dataset_specified_warning(self, *args):
+        for _ in range(3):
+            with self.assertLogs(BuscoConfig.logger, "WARNING"):
+                config = BuscoConfig.BuscoConfigMain(self.base_config, {})
+                config.configure()
+            self.assertEqual(config.get("busco_run", "auto-lineage-euk"), "False")
+            self.assertEqual(config.get("busco_run", "auto-lineage-prok"), "False")
+            self.assertEqual(config.get("busco_run", "auto-lineage"), "False")
 
     def test_mandatory_keys_check_log(self):
         with self.assertLogs(BuscoConfig.logger, 20):
@@ -300,7 +346,7 @@ class TestBuscoConfig(unittest.TestCase):
         config.main_out = os.path.join(
             config.get("busco_run", "out_path"), config.get("busco_run", "out")
         )
-        config._create_required_paths()
+        config._create_required_paths(config.main_out)
         output_dir = os.path.join(os.getcwd(), self.test_params["out"])
         self.assertTrue(os.path.exists(output_dir))
         shutil.rmtree(output_dir)
@@ -321,12 +367,11 @@ class TestBuscoConfig(unittest.TestCase):
                 config._check_allowed_keys()
 
     def test_out_value_check_invalid(self):
-        for str_format in ["/path/to/output", "output/"]:
-            self.test_params["out"] = str_format
-            with self.assertRaises(BuscoConfig.BatchFatalError):
-                config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
-                config.configure()
-                config._check_out_value()
+        self.test_params["out"] = "output/"
+        config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
+        config.configure()
+        config._check_out_value()
+        self.assertTrue(config.get("busco_run", "out") == "output")
 
     def test_out_value_check_valid(self):
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
@@ -358,6 +403,7 @@ class TestBuscoConfig(unittest.TestCase):
     @patch("__main__.BuscoConfig_unittests.BuscoConfig.logger.warning")
     def test_evalue_default(self, mock_logger):
         self.test_params["evalue"] = 0.001
+        self.test_params["lineage_dataset"] = "test"
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
         config.configure()
         config._check_evalue()
@@ -463,8 +509,6 @@ class TestBuscoConfig(unittest.TestCase):
     @patch.object(BuscoConfig.BuscoConfigMain, "_check_batch_mode")
     @patch.object(BuscoConfig.BuscoConfigMain, "_check_required_input_exists")
     @patch.object(BuscoConfig.BuscoConfigMain, "_expand_all_paths")
-    @patch.object(BuscoConfig.BuscoConfigMain, "_check_evalue")
-    @patch.object(BuscoConfig.BuscoConfigMain, "_check_limit_value")
     @patch.object(BuscoConfig.BuscoConfigMain, "_check_out_value")
     @patch.object(BuscoConfig.BuscoConfigMain, "_check_allowed_keys")
     @patch.object(BuscoConfig.BuscoConfigMain, "_create_required_paths")
@@ -477,8 +521,6 @@ class TestBuscoConfig(unittest.TestCase):
         mock_create_required_paths,
         mock_check_allowed_keys,
         mock_check_out_value,
-        mock_check_limit_value,
-        mock_check_evalue,
         mock_expand_all_paths,
         mock_check_input,
         mock_check_batch,
@@ -493,8 +535,6 @@ class TestBuscoConfig(unittest.TestCase):
         mock_create_required_paths.assert_called()
         mock_check_allowed_keys.assert_called()
         mock_check_out_value.assert_called()
-        mock_check_limit_value.assert_called()
-        mock_check_evalue.assert_called()
         mock_expand_all_paths.assert_called()
         mock_check_input.assert_called()
         mock_check_batch.assert_called()
@@ -515,7 +555,6 @@ class TestBuscoConfig(unittest.TestCase):
         self.test_params["datasets_version"] = "odb10"
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
         config.configure()
-        config.check_lineage_present()
         self.assertEqual(
             config.get("busco_run", "datasets_version"),
             self.test_params["datasets_version"],
@@ -525,9 +564,8 @@ class TestBuscoConfig(unittest.TestCase):
         self.test_params["lineage_dataset"] = "test_dataset_odb10"
         self.test_params["datasets_version"] = "odb11"
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
-        config.configure()
         with self.assertLogs(BuscoConfig.logger, level="WARNING"):
-            config.check_lineage_present()
+            config.configure()
         self.assertEqual(
             config.get("busco_run", "datasets_version"),
             self.test_params["lineage_dataset"].split("_")[-1],
@@ -538,7 +576,6 @@ class TestBuscoConfig(unittest.TestCase):
         self.test_params["datasets_version"] = "odb10"
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
         config.configure()
-        config.check_lineage_present()
         self.assertEqual(
             config.get("busco_run", "lineage_dataset"),
             "{}_{}".format(
@@ -551,25 +588,26 @@ class TestBuscoConfig(unittest.TestCase):
         self.test_params["lineage_dataset"] = "test_dataset"
         self.test_params["datasets_version"] = "odb11"
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
-        config.configure()
         with self.assertRaises(BuscoConfig.BatchFatalError):
-            config.check_lineage_present()
+            config.configure()
 
     def test_set_results_dirname(self):
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
         config.configure()
         test_dataset_path = "/path/to/lineage_dataset"
-        with patch("busco.BuscoConfig.BuscoConfig.set"):
+        with patch("busco.BuscoConfig.BaseConfig.set"):
             config.set_results_dirname(test_dataset_path)
             config.set.assert_called_with(
                 "busco_run", "lineage_results_dir", "run_lineage_dataset"
             )
 
+    @patch("busco.BuscoConfig.BuscoConfigAuto.add_mode_specific_parameters")
     @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset_config")
     @patch("busco.BuscoConfig.BuscoConfigAuto.download_lineage_file")
     @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
     @patch("busco.BuscoConfig.BuscoConfigAuto.set_results_dirname")
-    @patch("busco.BuscoConfig.BuscoConfig")
+    @patch("busco.BuscoConfig.BaseConfig")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
     @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
     def test_autoconfig_init_propagates_mainconfig(self, mock_propagate, *args):
         config = BuscoConfig.BuscoConfigMain(self.base_config, self.test_params)
@@ -577,39 +615,59 @@ class TestBuscoConfig(unittest.TestCase):
         BuscoConfig.BuscoConfigAuto(config, None)
         mock_propagate.assert_called_with(config)
 
+    @patch("busco.BuscoConfig.BuscoConfigAuto.add_mode_specific_parameters")
     @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset_config")
     @patch("busco.BuscoConfig.BuscoConfigAuto.download_lineage_file")
     @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
-    @patch("busco.BuscoConfig.BuscoConfig")
+    @patch("busco.BuscoConfig.BaseConfig")
     @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
     @patch("busco.BuscoConfig.BuscoConfigAuto.set_results_dirname")
     def test_autoconfig_init_sets_results_dirname(self, mock_set_dirname, *args):
         BuscoConfig.BuscoConfigAuto(None, "lineage")
         mock_set_dirname.assert_called_with("lineage")
 
     @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset")
-    @patch("busco.BuscoConfig.BuscoConfig")
+    @patch("busco.BuscoConfig.BaseConfig")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.add_mode_specific_parameters")
     @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
     @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
     def test_autoconfig_init_creates_paths(self, mock_create_paths, *args):
         BuscoConfig.BuscoConfigAuto(None, None)
         mock_create_paths.assert_called()
 
     @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset_config")
-    @patch("busco.BuscoConfig.BuscoConfig")
+    @patch("busco.BuscoConfig.BaseConfig")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.add_mode_specific_parameters")
     @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
     @patch("busco.BuscoConfig.BuscoConfigAuto.set_results_dirname")
     @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
     @patch("busco.BuscoConfig.BuscoConfigAuto.download_lineage_file")
     def test_autoconfig_init_downloads_lineage(self, mock_download_lineage, *args):
         BuscoConfig.BuscoConfigAuto(None, "lineage")
         mock_download_lineage.assert_called_with("lineage")
 
-    @patch("busco.BuscoConfig.BuscoConfig")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset_config")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.download_lineage_file")
+    @patch("busco.BuscoConfig.BaseConfig")
+    @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.set_results_dirname")
+    @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.add_mode_specific_parameters")
+    def test_autoconfig_init_adds_specific_parameters(self, mock_add_parameters, *args):
+        BuscoConfig.BuscoConfigAuto(None, "lineage")
+        mock_add_parameters.assert_called()
+
+    @patch("busco.BuscoConfig.BaseConfig")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.add_mode_specific_parameters")
     @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
     @patch("busco.BuscoConfig.BuscoConfigAuto.set_results_dirname")
     @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
     @patch("busco.BuscoConfig.BuscoConfigAuto.download_lineage_file")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
     @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset_config")
     def test_autoconfig_init_loads_lineage_config(self, mock_load_dataset, *args):
         BuscoConfig.BuscoConfigAuto(None, None)
@@ -618,11 +676,13 @@ class TestBuscoConfig(unittest.TestCase):
     @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
     @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
     @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset")
-    @patch("busco.BuscoConfig.BuscoConfig.__init__")
+    @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
+    @patch("busco.BuscoConfig.BaseConfig.__init__")
     def test_autoconfig_init_calls_super(self, mock_config_parent, *args):
         BuscoConfig.BuscoConfigAuto(None, None)
         mock_config_parent.assert_called()
 
+    @patch("busco.BuscoConfig.BuscoConfigAuto.add_mode_specific_parameters")
     @patch("busco.BuscoConfig.BuscoConfigAuto._create_required_paths")
     @patch("busco.BuscoConfig.BuscoConfigAuto.download_lineage_file")
     @patch("busco.BuscoConfig.BuscoConfigAuto.load_dataset_config")
@@ -639,7 +699,7 @@ class TestBuscoConfig(unittest.TestCase):
     @patch("busco.BuscoConfig.BuscoConfigAuto._propagate_config")
     @patch("busco.BuscoConfig.BuscoConfigAuto.set_results_dirname")
     @patch("busco.BuscoConfig.BuscoConfigAuto.get", return_value="test")
-    @patch("busco.BuscoConfig.BuscoConfig._create_required_paths")
+    @patch("busco.BuscoConfig.BaseConfig._create_required_paths")
     def test_autolineage_create_path_method_calls_parent(
         self, mock_create_paths, *args
     ):
